<?php
namespace App\JsonRpc;


use Hyperf\RpcServer\Annotation\RpcService;
use App\Tools\Result;
use App\Model\GlobalTable;
use Hyperf\DbConnection\Db;
use App\Model\GlobalTableField;
use App\Model\GlobalTableFieldType;
use App\Model\GlobalTableFieldValue;
use Hyperf\Di\Annotation\Inject;
use Hyperf\Redis\Redis;
#[RpcService(name: "FormService", protocol: "jsonrpc-http", server: "jsonrpc-http")]
class FormService implements FormServiceInterface
{

    #[Inject]
    protected Redis $redis;
    /**
     * 添加全局表单
     * @param array $data
     * @return array|mixed
     */
    public function addGlobalTable(array $data): array
    {
        // 过滤掉空值
        $data = array_filter($data, function($value) {
            return !empty($value);
        });

        // 检查是否已存在相同名称的记录
        $globalTable = GlobalTable::on('global')->where(['table'=> $data['table']])->first();
        if (empty($globalTable)) {
            $id = GlobalTable::on('global')->insertGetId($data);
            //给global库创建表 表名为$data['table'] 的值，并初始化一个字段id，类型为int，自增，主键，再初始化一个字段title,类型为varchar，长度为255
            Db::connection('global')->statement("CREATE TABLE IF NOT EXISTS `{$data['table']}` (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `title` varchar(255) DEFAULT NULL,
                `updated_at` timestamp NULL DEFAULT CURRENT_TIMESTAMP COMMENT '更新时间',
                `created_at` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP COMMENT '创建时间',
                PRIMARY KEY (`id`)
            )");
            //给GlobalTableField表初始化数据
            GlobalTableField::on('global')->insert([
                [
                    'table_id' => $id,
                    'field_name' => 'id',
                    'title' => '编号',
                    'field_type' => '1',
                    'length' => 255,
                    'sort' => 0,
                    'is_check' => 1,
                    'admin_display' => 1,
                    'home_display' => 1,
                    'disable_del' => 1,
                ],
                [
                    'table_id' => $id,
                    'field_name' => 'title',
                    'title' => '标题',
                    'field_type' => '1',
                    'length' => 255,
                    'sort' => 0,
                    'is_check' => 1,
                    'admin_display' => 1,
                    'home_display' => 1,
                    'disable_del' => 1,
                ],
                [
                    'table_id' => $id,
                    'field_name' => 'updated_at',
                    'title' => '更新时间',
                    'field_type' => '1',
                    'length' => 255,
                    'sort' => 999,
                    'is_check' => 1,
                    'admin_display' => 1,
                    'home_display' => 0,
                    'disable_del' => 1,
                ],
                [
                    'table_id' => $id,
                    'field_name' => 'created_at',
                    'title' => '创建时间',
                    'field_type' => '1',
                    'length' => 255,
                    'sort' => 1000,
                    'is_check' => 1,
                    'admin_display' => 1,
                    'home_display' => 0,
                    'disable_del' => 1,
                ],
                
            ]);
 
            return Result::success('添加成功');
        }
        return Result::error('表单已存在');
    }
    /**
     * 获取全局表单列表
     * @param array $data
     * @return array
     */
    public function getGlobalTableList(array $data): array
    {
        try {
            // 构建查询
            $query = GlobalTable::query()
                ->when(!empty($data['website_id']), function($q) use ($data) {
                    return $q->where('website_id', $data['website_id']);
                })
                ->when(!empty($data['name']), function($q) use ($data) {
                    return $q->where('name', 'like', '%' . $data['name'] . '%');
                });
            
            // 分页参数
            $page = (int)($data['page'] ?? 1);
            $pageSize = (int)($data['pageSize'] ?? 10);
            
            // 先获取总数
            $total = $query->count();
            
            // 获取数据
            $list = $query->orderBy('updated_at', 'desc')
                         ->offset(($page - 1) * $pageSize)
                         ->limit($pageSize)
                         ->get();
            
            // 如果没有数据，直接返回空结果
            if ($list->isEmpty()) {
                return Result::success([
                    'list' => [],
                    'total' => $total,
                    'page' => $page,
                    'pageSize' => $pageSize
                ]);
            }
            
            // 获取并合并 website 数据
            $websites = Db::connection('secondary')
                ->table('website')
                ->whereIn('id', $list->pluck('website_id'))
                ->pluck('website_name', 'id');
            
            // 合并数据并返回
            return Result::success([
                'list' => $list->map(function($item) use ($websites) {
                    $item->website_name = $websites[$item->website_id] ?? '';
                    return $item;
                }),
                'total' => $total,
                'page' => $page,
                'pageSize' => $pageSize
            ]);
            
        } catch (\Throwable $e) {
            return Result::error('查询失败：' . $e->getMessage());
        }
    }
    /**
     * 获取全局表单
     * @param array $data
     * @return array
     */
    public function getGlobalTable(array $data): array
    {
        $globalTable = GlobalTable::where('id',$data['id'])->first();
        return Result::success($globalTable);
    }
    /**
     * 修改全局表单
     * @param array $data
     * @return array
     */
    public function upGlobalTable(array $data): array   
    {
        $globalTable = GlobalTable::where('id',$data['id'])->first();
        if(empty($globalTable)){
            return Result::error('表单不存在');
        }   
        $data = array_filter($data,function($value){
            return $value !== null;
        });
        GlobalTable::where(['id'=>$data['id']])->update($data);
        return Result::success('修改成功');
    }   
    /**
     * 删除全局表单
     * @param array $data
     * @return array
     */
    public function delGlobalTable(array $data): array          
    {
        $globalTable = GlobalTable::where('id', $data['id'])->first();
        if(empty($globalTable)){
            return Result::error('表单不存在');
        }
        //删除global库的表
        Db::connection('global')->statement("DROP TABLE IF EXISTS `{$globalTable->table}`");
        //删除GlobalTableField表中table_id为$data['id']的数据
        GlobalTableField::where('table_id', $data['id'])->delete();
        GlobalTable::where('id', $data['id'])->delete();
        return Result::success('删除成功');
    }
    /**
     * 获取表单字段列表
     * @param array $data
     * @return array
     */
    public function getGlobalTableFieldList(array $data): array
    {
        $fields = GlobalTableField::from('global_table_field as a')
            ->join('global_table_field_type as b', 'a.field_type', '=', 'b.id')
            ->where('a.table_id', $data['id'])
            ->whereNotIn('a.field_name',['id','created_at','updated_at'])
            ->orderBy('a.sort', 'asc')
            ->select(
            // global_table_field 所有字段
                'a.*',
                // global_table_field_type 字段
                'b.type_name',
                'b.type_name_alias',
                'b.field_type as type_definition'
            )
            ->get();
        return Result::success($fields);
    }
    /**
     * 获取表单字段
     * @param array $data
     * @return array    
     */
    public function getGlobalTableField(array $data): array
    {
        $globalTableField = GlobalTableField::where('id',$data['id'])->first();
        return Result::success($globalTableField);
    }
    /**
     * 添加表单字段
     * @param array $data
     * @return array
     * @throws \Throwable
     */
    public function addGlobalTableField(array $data): array
    {
        $globalTable = GlobalTable::where('id',$data['table_id'])->first(); 
        if(empty($globalTable)){
            return Result::error('表单不存在');     
        }
        //检查是否已存在相同名称的记录
        $globalTableField = GlobalTableField::where(['table_id'=>$data['table_id'],'field_name'=>$data['field_name']])->first();    
        if(!empty($globalTableField)){
            return Result::error('字段已存在');
        }

        // 检查表中是否已存在该列
        try {
            $columns = Db::connection('global')->select("SHOW COLUMNS FROM `{$globalTable->table}`");
            $columnNames = array_column($columns, 'Field');
            if (in_array($data['field_name'], $columnNames)) {
                return Result::error('字段名已存在于数据表中');
            }
        } catch (\Throwable $e) {
            return Result::error('检查字段是否存在时出错：' . $e->getMessage());
        }

        $globalTableFieldTypeInfo = GlobalTableFieldType::where('id',$data['field_type'])->first();
        //给global库的表添加字段
        Db::connection('global')->statement("ALTER TABLE `{$globalTable->table}` ADD COLUMN `{$data['field_name']}` {$globalTableFieldTypeInfo['field_type']}({$data['length']}) COMMENT '{$data['title']}'"); 
        //给GlobalTableField表添加数据
        GlobalTableField::create($data);
        return Result::success('添加成功');
    }
    /**
     * 修改表单字段
     * @param array $data
     * @return array
     * @throws \Throwable
     */
    public function upGlobalTableField(array $data): array
    {
        $globalTable = GlobalTable::where('id',$data['table_id'])->first(); 
        if(empty($globalTable)){
            return Result::error('表单不存在');
        }
        //检查是否已存在相同名称的记录
        $globalTableField = GlobalTableField::where(['table_id'=>$data['table_id'],'field_name'=>$data['field_name']])->first();
        if(!empty($globalTableField) && $globalTableField->id != $data['id']){
            return Result::error('字段已存在');
        }
        $globalTableFieldTypeInfo = GlobalTableFieldType::where('id',$data['field_type'])->first();
        //给global库的表修改字段
        Db::connection('global')->statement("ALTER TABLE `{$globalTable->table}` CHANGE COLUMN `{$data['field_name']}` `{$data['field_name']}` {$globalTableFieldTypeInfo['field_type']}({$data['length']}) COMMENT '{$data['title']}'"); 
        //给GlobalTableField表修改数据
        GlobalTableField::where('id',$data['id'])->update($data);
        return Result::success('修改成功');
    }
    /**
     * 删除表单字段
     * @param array $data
     * @return array
     * @throws \Throwable
     */
    public function delGlobalTableField(array $data): array
    {
        $tableFieldInfo = GlobalTableField::where('id',$data['id'])->first(); 
        if(empty($tableFieldInfo)){
            return Result::error('字段不存在');
        }
        $globalTable = GlobalTable::where('id',$tableFieldInfo['table_id'])->first(); 
        if(empty($globalTable)){
            return Result::error('表单不存在');
        }
        //给global库的表删除字段
        Db::connection('global')->statement("ALTER TABLE `{$globalTable->table}` DROP COLUMN `{$tableFieldInfo['field_name']}`"); 
        //给GlobalTableField表删除数据
        GlobalTableField::where('id',$data['id'])->delete();
        return Result::success('删除成功');
    }
    /**
     * 获取自定义生成表里面的数据
     */
    public function getGlobalTableData(array $data): array
    {
        try {
            //查询global库的表GlobalTableField模型中table_id为$data['id']的数据条件为admin_display=1的数据
            $globalTableFields = GlobalTableField::where(['table_id'=>$data['id'],'admin_display'=>1])->get();
            $globalTableFields->transform(function ($field) {
                $optionValue = [];
                $optionStr = trim((string) $field->option);
                if (!empty($optionStr)) {
                    $options = explode("\n", $optionStr);
                    foreach ($options as $option) {
                        $parts = explode('|', $option);
                        if (count($parts) === 2) {
                            $optionValue[$parts[1]] = $parts[0];
                        }
                    }
                }
                $field->option_value = $optionValue;
                return $field;
            });
            
            //查询global库的表GlobalTable模型中id为$data['id']的数据
            $globalTable = GlobalTable::where('id',$data['id'])->first(); 
            if(empty($globalTable)){
                return Result::error('表单不存在');
            }

            // 构建查询
            $query = Db::connection('global')->table($globalTable->table);
            
            // 添加title模糊搜索
            if (!empty($data['title'])) {
                $query->where('title', 'like', '%' . $data['title'] . '%');
            }

            // 分页参数
            $page = (int)($data['page'] ?? 1);
            $pageSize = (int)($data['pageSize'] ?? 10);
            
            // 先获取总数
            $total = $query->count();
            
            // 获取分页数据
           $list = $query->select($globalTableFields->pluck('field_name')->toArray())
             ->orderBy('id', 'desc')
             ->offset(($page - 1) * $pageSize)
             ->limit($pageSize)
             ->get();

            // 转换字符串数组为实际数组
            $list->transform(function ($item) use ($globalTableFields) {
                foreach ($globalTableFields as $field) {
                    $fieldName = $field->field_name;
                    if (isset($item->$fieldName) && is_string($item->$fieldName)) {
                        // 尝试解析JSON格式的字符串
                        $decoded = json_decode($item->$fieldName, true);
                        if (json_last_error() === JSON_ERROR_NONE) {
                            $item->$fieldName = $decoded;
                        }
                        // 如果是简单的逗号分隔字符串，也可以转换为数组
                        elseif (strpos($item->$fieldName, ',') !== false) {
                            $item->$fieldName = array_map('trim', explode(',', $item->$fieldName));
                        }
                    }
                }
                return $item;
            });


            return Result::success([
                'tableFields' => $globalTableFields,
                'list' => $list,
                'total' => $total,
                'page' => $page,
                'pageSize' => $pageSize
            ]);
        } catch (\Throwable $e) {
            return Result::error('查询失败：' . $e->getMessage());
        }
    }
    /**
     * 获取字段类型列表
     */
    public function getGlobalTableFieldTypeList(array $data): array
    {
        try {
            $list =  Db::connection('global')->table("global_table_field_type")->get();
            return Result::success($list);
        }catch (\Throwable $e){
            return Result::error('查询失败：' . $e->getMessage());
        }
    }
    /**
     * 删除自定义表里面的某一条数据
     */
    public function delGlobalTableData(array $data): array
    {
        try {
            //查询global库的表GlobalTable模型中id为$data['id']的数据
            $globalTable = GlobalTable::where('id',$data['table_id'])->first();
            if(empty($globalTable)){
                return Result::error('表单不存在');
            }
            // 构建查询
            $query = Db::connection('global')->table($globalTable->table);
            $query->where('id',$data['id']);
            $query->delete();
            return Result::success('删除成功');
        }catch (\Throwable $e){
            return Result::error('删除失败：' . $e->getMessage());
        }
    }
    /**
     * 修改自定义表里面的某一条数据
     */
    public function updateGlobalTableData(array $data): array
    {
        try {
            $globalTable = GlobalTable::where('id', $data['table_id'])->first();
            if (empty($globalTable)) {
                return Result::error('表单不存在');
            }
            unset($data['table_id']);
            // 转换数组字段为字符串
            $data = array_map(function ($value) {
                if (is_array($value)) {
                    return json_encode($value, JSON_UNESCAPED_UNICODE);
                }
                return $value;
            }, $data);
            $query = Db::connection('global')->table($globalTable->table);
            $query->where('id', $data['id'])->update($data);
            return Result::success('修改成功');
        } catch (\Throwable $e) {
            return Result::error('修改失败：' . $e->getMessage());
        }
    }

    /**
     * 查看自定义表里面的某条数据
     */
    public function getGlobalTableDataById(array $data): array
    {
        try {
            //查询
            $globalTable = GlobalTable::where('id',$data['table_id'])->first();
            if(empty($globalTable)){
                return Result::error('表单不存在');
            }
            $query =  Db::connection('global')->table($globalTable->table);
            $query->where('id',$data['id']);
            $dataInfo = $query->first();
            $globalTableField = GlobalTableField::where(['table_id'=>$data['table_id']])->get();
            $globalTableField->transform(function ($field) {
                $optionValue = [];
                $optionStr = trim((string) $field->option);
                if (!empty($optionStr)) {
                    $options = explode("\n", $optionStr);
                    foreach ($options as $option) {
                        $parts = explode('|', $option);
                        if (count($parts) === 2) {
                            $optionValue[$parts[1]] = $parts[0];
                        }
                    }
                }
                $field->option_value = $optionValue;
                return $field;
            });
            $return = [
                'data'=>$dataInfo,
                'tableFields'=>$globalTableField->toArray(),
            ];
            return Result::success($return);
        }catch (\Throwable $e){
            return Result::error('查询失败：' . $e->getMessage());
        }
    }
    /**
     * 查询网站下表单自定义字段
     */
    public function getWebGlobalTableFieldList(array $data): array
    {
        try {
            $globalTable = GlobalTable::where('id',$data['table_id'])->first();
            $fields = GlobalTableField::from('global_table_field as a')
                ->join('global_table_field_type as b', 'a.field_type', '=', 'b.id')
                ->where('a.table_id', $data['table_id'])
                ->where('a.field_name',"<>",'id')
                ->where('a.home_display', 1)
                ->orderBy('a.sort', 'asc')
                ->select(
                // global_table_field 所有字段
                    'a.*',
                    // global_table_field_type 字段
                    'b.type_name',
                    'b.type_name_alias',
                    'b.field_type as type_definition'
                )
                ->get();
            $fields->transform(function ($field) {
                    $optionValue = [];
                    $optionStr = trim((string) $field->option);
                    if (!empty($optionStr)) {
                        $options = explode("\n", $optionStr);
                        foreach ($options as $option) {
                            $parts = explode('|', $option);
                            if (count($parts) === 2) {
                                $optionValue[$parts[1]] = $parts[0];
                            }
                        }
                    }
                    $field->option_value = $optionValue;
                    return $field;
                });
            $config = new \EasySwoole\VerifyCode\Config();
            $code = new \EasySwoole\VerifyCode\VerifyCode($config);
            $img_code = '';
            $characters = '0123456789';
            $charLength = strlen($characters);
            for ($i = 0; $i < 4; $i++) {
                $img_code .= $characters[rand(0, $charLength - 1)];
            }
            //重写验证码
            $result = $code->DrawCode((string)$img_code);
            $img_code = $result->getImageCode();
            var_dump("验证码：",$img_code);
            $code_uniqid = uniqid("code");
            //写入缓存 用于其他方法验证 并且设置过期时间
            $this->redis->set($code_uniqid,$img_code,60000);
            $rep = [
                'fields' => $fields,
                'table'=>$globalTable,
            ];
            if($globalTable->is_code){
                $rep['code']['code_uniqid'] = $code_uniqid;
                $rep['code']['img'] = $result->getImageBase64();
            }
            return Result::success($rep);
        }catch (\Throwable $e){
            return Result::error('查询失败：' . $e->getMessage());
        }
    }

    /**
     * web端创建数据
     * @param array $data
     * @return array
     */
    public function addWebGlobalTableData(array $data): array
    {
        try {
            $globalTable = GlobalTable::where('id',$data['otherData']['table_id'])->first();
            if($globalTable->is_code){
                if(empty($data['otherData']['code'])){
                    return Result::error('请输入验证码');
                }
                $code = $this->redis->get($data['otherData']['code_uniqid']);   
                if(empty($code)){
                    return Result::error('验证码已过期');
                }
                if($data['otherData']['code'] != $code){
                    return Result::error('验证码错误');
                }
            }
            if(empty($globalTable)){
                return Result::error('表单不存在');
            }
            $query = Db::connection('global')->table($globalTable->table);
            $data['data'] = array_map(function ($value) {
                if (is_array($value)) {
                    return json_encode($value, JSON_UNESCAPED_UNICODE);
                }
                return $value;
            }, $data['data']);
            $query->insert($data['data']);
            return Result::success([]);
        }catch (\Throwable $e){
            return Result::error('添加失败：' . $e->getMessage());
        }
    }
}